@@ -10,7 +10,9 @@ use ahash::AHashSet;
 use url::{ParseError, SyntaxViolation, Url};
 
 use crate::build_tools::{is_strict, py_schema_err};
+use crate::errors::ToErrorValue;
 use crate::errors::{ErrorType, ErrorTypeDefaults, ValError, ValResult};
+use crate::input::downcast_python_input;
 use crate::input::Input;
 use crate::tools::SchemaDict;
 use crate::url::{schema_is_special, PyMultiHostUrl, PyUrl};
@@ -37,8 +39,8 @@ impl BuildValidator for UrlValidator {
     const EXPECTED_TYPE: &'static str = "url";
 
     fn build(
-        schema: &PyDict,
-        config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        config: Option<&Bound<'_, PyDict>>,
         _definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let (allowed_schemes, name) = get_allowed_schemas(schema, Self::EXPECTED_TYPE)?;
@@ -60,16 +62,16 @@ impl BuildValidator for UrlValidator {
 impl_py_gc_traverse!(UrlValidator {});
 
 impl Validator for UrlValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
-        let mut lib_url = self.get_url(input, state.strict_or(self.strict))?;
+        let mut either_url = self.get_url(input, state.strict_or(self.strict))?;
 
         if let Some((ref allowed_schemes, ref expected_schemes_repr)) = self.allowed_schemes {
-            if !allowed_schemes.contains(lib_url.scheme()) {
+            if !allowed_schemes.contains(either_url.url().scheme()) {
                 let expected_schemes = expected_schemes_repr.clone();
                 return Err(ValError::new(
                     ErrorType::UrlScheme {
@@ -82,7 +84,7 @@ impl Validator for UrlValidator {
         }
 
         match check_sub_defaults(
-            &mut lib_url,
+            &mut either_url,
             self.host_required,
             &self.default_host,
             self.default_port,
@@ -91,7 +93,7 @@ impl Validator for UrlValidator {
             Ok(()) => {
                 // Lax rather than strict to preserve V2.4 semantic that str wins over url in union
                 state.floor_exactness(Exactness::Lax);
-                Ok(PyUrl::new(lib_url).into_py(py))
+                Ok(either_url.into_py(py))
             }
             Err(error_type) => Err(ValError::new(error_type, input)),
         }
@@ -103,7 +105,7 @@ impl Validator for UrlValidator {
 }
 
 impl UrlValidator {
-    fn get_url<'s, 'data>(&'s self, input: &'data impl Input<'data>, strict: bool) -> ValResult<Url> {
+    fn get_url<'py>(&self, input: &(impl Input<'py> + ?Sized), strict: bool) -> ValResult<EitherUrl<'py>> {
         match input.validate_str(strict, false) {
             Ok(val_match) => {
                 let either_str = val_match.into_inner();
@@ -112,20 +114,19 @@ impl UrlValidator {
 
                 self.check_length(input, url_str)?;
 
-                parse_url(url_str, input, strict)
+                parse_url(url_str, input, strict).map(EitherUrl::Rust)
             }
             Err(_) => {
                 // we don't need to worry about whether the url was parsed in strict mode before,
                 // even if it was, any syntax errors would have been fixed by the first validation
-                if let Some(py_url) = input.input_as_url() {
-                    let lib_url = py_url.into_url();
-                    self.check_length(input, lib_url.as_str())?;
-                    Ok(lib_url)
-                } else if let Some(multi_host_url) = input.input_as_multi_host_url() {
-                    let url_str = multi_host_url.__str__();
+                if let Some(py_url) = downcast_python_input::<PyUrl>(input) {
+                    self.check_length(input, py_url.get().url().as_str())?;
+                    Ok(EitherUrl::Py(py_url.clone()))
+                } else if let Some(multi_host_url) = downcast_python_input::<PyMultiHostUrl>(input) {
+                    let url_str = multi_host_url.get().__str__();
                     self.check_length(input, &url_str)?;
 
-                    parse_url(&url_str, input, strict)
+                    parse_url(&url_str, input, strict).map(EitherUrl::Rust)
                 } else {
                     Err(ValError::new(ErrorTypeDefaults::UrlType, input))
                 }
@@ -133,7 +134,7 @@ impl UrlValidator {
         }
     }
 
-    fn check_length<'s, 'data>(&self, input: &'data impl Input<'data>, url_str: &str) -> ValResult<()> {
+    fn check_length<'py>(&self, input: &(impl Input<'py> + ?Sized), url_str: &str) -> ValResult<()> {
         if let Some(max_length) = self.max_length {
             if url_str.len() > max_length {
                 return Err(ValError::new(
@@ -149,6 +150,39 @@ impl UrlValidator {
     }
 }
 
+enum EitherUrl<'py> {
+    Py(Bound<'py, PyUrl>),
+    Rust(Url),
+}
+
+impl EitherUrl<'_> {
+    fn into_py(self, py: Python) -> PyObject {
+        match self {
+            EitherUrl::Py(py_url) => py_url.into_py(py),
+            EitherUrl::Rust(rust_url) => PyUrl::new(rust_url).into_py(py),
+        }
+    }
+}
+
+impl CopyFromPyUrl for EitherUrl<'_> {
+    fn url(&self) -> &Url {
+        match self {
+            EitherUrl::Py(py_url) => py_url.get().url(),
+            EitherUrl::Rust(rust_url) => rust_url,
+        }
+    }
+
+    fn url_mut(&mut self) -> &mut Url {
+        if let EitherUrl::Py(py_url) = self {
+            *self = EitherUrl::Rust(py_url.get().url().clone());
+        }
+        match self {
+            EitherUrl::Py(_) => unreachable!(),
+            EitherUrl::Rust(rust_url) => rust_url,
+        }
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct MultiHostUrlValidator {
     strict: bool,
@@ -165,8 +199,8 @@ impl BuildValidator for MultiHostUrlValidator {
     const EXPECTED_TYPE: &'static str = "multi-host-url";
 
     fn build(
-        schema: &PyDict,
-        config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        config: Option<&Bound<'_, PyDict>>,
         _definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let (allowed_schemes, name) = get_allowed_schemas(schema, Self::EXPECTED_TYPE)?;
@@ -194,16 +228,16 @@ impl BuildValidator for MultiHostUrlValidator {
 impl_py_gc_traverse!(MultiHostUrlValidator {});
 
 impl Validator for MultiHostUrlValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         let mut multi_url = self.get_url(input, state.strict_or(self.strict))?;
 
         if let Some((ref allowed_schemes, ref expected_schemes_repr)) = self.allowed_schemes {
-            if !allowed_schemes.contains(multi_url.scheme()) {
+            if !allowed_schemes.contains(multi_url.url().scheme()) {
                 let expected_schemes = expected_schemes_repr.clone();
                 return Err(ValError::new(
                     ErrorType::UrlScheme {
@@ -215,7 +249,7 @@ impl Validator for MultiHostUrlValidator {
             }
         }
         match check_sub_defaults(
-            multi_url.mut_lib_url(),
+            &mut multi_url,
             self.host_required,
             &self.default_host,
             self.default_port,
@@ -236,7 +270,7 @@ impl Validator for MultiHostUrlValidator {
 }
 
 impl MultiHostUrlValidator {
-    fn get_url<'s, 'data>(&'s self, input: &'data impl Input<'data>, strict: bool) -> ValResult<PyMultiHostUrl> {
+    fn get_url<'py>(&self, input: &(impl Input<'py> + ?Sized), strict: bool) -> ValResult<EitherMultiHostUrl<'py>> {
         match input.validate_str(strict, false) {
             Ok(val_match) => {
                 let either_str = val_match.into_inner();
@@ -245,18 +279,20 @@ impl MultiHostUrlValidator {
 
                 self.check_length(input, || url_str.len())?;
 
-                parse_multihost_url(url_str, input, strict)
+                parse_multihost_url(url_str, input, strict).map(EitherMultiHostUrl::Rust)
             }
             Err(_) => {
                 // we don't need to worry about whether the url was parsed in strict mode before,
                 // even if it was, any syntax errors would have been fixed by the first validation
-                if let Some(multi_url) = input.input_as_multi_host_url() {
-                    self.check_length(input, || multi_url.__str__().len())?;
-                    Ok(multi_url)
-                } else if let Some(py_url) = input.input_as_url() {
-                    let lib_url = py_url.into_url();
-                    self.check_length(input, || lib_url.as_str().len())?;
-                    Ok(PyMultiHostUrl::new(lib_url, None))
+                if let Some(multi_url) = downcast_python_input::<PyMultiHostUrl>(input) {
+                    self.check_length(input, || multi_url.get().__str__().len())?;
+                    Ok(EitherMultiHostUrl::Py(multi_url.clone()))
+                } else if let Some(py_url) = downcast_python_input::<PyUrl>(input) {
+                    self.check_length(input, || py_url.get().url().as_str().len())?;
+                    Ok(EitherMultiHostUrl::Rust(PyMultiHostUrl::new(
+                        py_url.get().url().clone(),
+                        None,
+                    )))
                 } else {
                     Err(ValError::new(ErrorTypeDefaults::UrlType, input))
                 }
@@ -264,7 +300,7 @@ impl MultiHostUrlValidator {
         }
     }
 
-    fn check_length<'s, 'data, F>(&self, input: &'data impl Input<'data>, func: F) -> ValResult<()>
+    fn check_length<'py, F>(&self, input: &(impl Input<'py> + ?Sized), func: F) -> ValResult<()>
     where
         F: FnOnce() -> usize,
     {
@@ -283,9 +319,42 @@ impl MultiHostUrlValidator {
     }
 }
 
-fn parse_multihost_url<'url, 'input>(
-    url_str: &'url str,
-    input: &'input impl Input<'input>,
+enum EitherMultiHostUrl<'py> {
+    Py(Bound<'py, PyMultiHostUrl>),
+    Rust(PyMultiHostUrl),
+}
+
+impl EitherMultiHostUrl<'_> {
+    fn into_py(self, py: Python) -> PyObject {
+        match self {
+            EitherMultiHostUrl::Py(py_multi_url) => py_multi_url.into_py(py),
+            EitherMultiHostUrl::Rust(rust_multi_url) => rust_multi_url.into_py(py),
+        }
+    }
+}
+
+impl CopyFromPyUrl for EitherMultiHostUrl<'_> {
+    fn url(&self) -> &Url {
+        match self {
+            EitherMultiHostUrl::Py(py_multi_url) => py_multi_url.get().lib_url(),
+            EitherMultiHostUrl::Rust(rust_multi_url) => rust_multi_url.lib_url(),
+        }
+    }
+
+    fn url_mut(&mut self) -> &mut Url {
+        if let EitherMultiHostUrl::Py(py_multi_url) = self {
+            *self = EitherMultiHostUrl::Rust(py_multi_url.get().clone());
+        }
+        match self {
+            EitherMultiHostUrl::Py(_) => unreachable!(),
+            EitherMultiHostUrl::Rust(rust_multi_url) => rust_multi_url.mut_lib_url(),
+        }
+    }
+}
+
+fn parse_multihost_url<'py>(
+    url_str: &str,
+    input: &(impl Input<'py> + ?Sized),
     strict: bool,
 ) -> ValResult<PyMultiHostUrl> {
     macro_rules! parsing_err {
@@ -402,7 +471,7 @@ fn parse_multihost_url<'url, 'input>(
     }
 }
 
-fn parse_url<'url, 'input>(url_str: &'url str, input: &'input impl Input<'input>, strict: bool) -> ValResult<Url> {
+fn parse_url(url_str: &str, input: impl ToErrorValue, strict: bool) -> ValResult<Url> {
     if url_str.is_empty() {
         return Err(ValError::new(
             ErrorType::UrlParsing {
@@ -465,7 +534,7 @@ fn parse_url<'url, 'input>(url_str: &'url str, input: &'input impl Input<'input>
 
 /// check host_required and substitute `default_host`, `default_port` & `default_path` if they aren't set
 fn check_sub_defaults(
-    lib_url: &mut Url,
+    url: &mut impl CopyFromPyUrl,
     host_required: bool,
     default_host: &Option<String>,
     default_port: Option<u16>,
@@ -475,9 +544,9 @@ fn check_sub_defaults(
         error: e.to_string(),
         context: None,
     };
-    if !lib_url.has_host() {
-        if let Some(ref default_host) = default_host {
-            lib_url.set_host(Some(default_host)).map_err(map_parse_err)?;
+    if let Some(ref default_host) = default_host {
+        if !url.url().has_host() {
+            url.url_mut().set_host(Some(default_host)).map_err(map_parse_err)?;
         } else if host_required {
             return Err(ErrorType::UrlParsing {
                 error: ParseError::EmptyHost.to_string(),
@@ -485,24 +554,31 @@ fn check_sub_defaults(
             });
         }
     }
-    if lib_url.port().is_none() {
-        if let Some(default_port) = default_port {
-            lib_url
+    if let Some(default_port) = default_port {
+        if url.url().port().is_none() {
+            url.url_mut()
                 .set_port(Some(default_port))
                 .map_err(|()| map_parse_err(ParseError::EmptyHost))?;
         }
     }
     if let Some(ref default_path) = default_path {
-        let path = lib_url.path();
+        let path = url.url().path();
         if path.is_empty() || path == "/" {
-            lib_url.set_path(default_path);
+            url.url_mut().set_path(default_path);
         }
     }
     Ok(())
 }
 
-fn get_allowed_schemas(schema: &PyDict, name: &'static str) -> PyResult<(AllowedSchemas, String)> {
-    match schema.get_as::<&PyList>(intern!(schema.py(), "allowed_schemes"))? {
+/// Abstraction to create a new Url only when necessary if the existing Url is a PyUrl
+/// and needs to be updated with new defaults
+trait CopyFromPyUrl {
+    fn url(&self) -> &Url;
+    fn url_mut(&mut self) -> &mut Url;
+}
+
+fn get_allowed_schemas(schema: &Bound<'_, PyDict>, name: &'static str) -> PyResult<(AllowedSchemas, String)> {
+    match schema.get_as::<Bound<'_, PyList>>(intern!(schema.py(), "allowed_schemes"))? {
         Some(list) => {
             if list.is_empty() {
                 return py_schema_err!("`allowed_schemes` should have length > 0");
